/****************************************
Autor: Rachmaninov
Versión: 1.0
****************************************/

// Qt Core

#include <QString>
#include <QStack>
#include <QFile>
#include <QMap>

// Qt GUI 

#include <QApplication>
#include <QMessageBox>
#include <QClipboard>
#include <QMimeData>

// Qt XML

#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QXmlStreamWriter>

#include <QLabel>
#include <QVBoxLayout>


class Exception {
public:
	Exception(QString message) {
		this->message = message;
	}

	inline QString getMessage() {
		return message;
	}

private:
	QString message;
};

struct XmlTag {
public:
	QString name;
	QXmlStreamAttributes attrs;
};

struct TextStyle {
public:

	enum {
		Bold = 0x01,
		Italic = 0x02,
		Underline = 0x04
	};

	int style;
};

class OdtHtml {
public:

	OdtHtml() {
		sw = new QXmlStreamWriter(&html);
	}

	~OdtHtml() {
		delete sw;

		QList<TextStyle*> values = mapStyles.values();
		for (int i=0; i<values.size(); i++) {
			delete values.at(i);
		}
	}
	
	void convert(QString& fileName);

	inline QString& getHtml() {
		return html;
	}	

private:
	QString html;
	QXmlStreamWriter* sw;
	QMap<QString, TextStyle*> mapStyles;

	void processStyles(const QDomElement& rootElement);

	void processOfficeText(const QDomElement& rootElement);

	void processOfficeTextChild(const QDomNode& node);

	void processStyleNode(const QDomNode& node);
};

void OdtHtml::convert(QString& fileName) {

	QFile file(fileName);
	if (!file.open(QIODevice::ReadOnly)) {
		throw Exception("Error al abrir el archivo.");
	} 
	
	QByteArray content = file.readAll();

	QDomDocument doc;
	int line, col;
	QString errorMessage;

	if (!doc.setContent(content, &errorMessage, &line, &col)) {
		QString message = QApplication::tr("Error en (%1,%2): %3").arg(line).arg(col).arg(errorMessage);
		throw Exception(message);
	}

	QDomElement rootElement = doc.documentElement();

	processStyles(rootElement);
	processOfficeText(rootElement);

	file.close();
}

void OdtHtml::processStyles(const QDomElement& rootElement) {
	QDomNodeList list = rootElement.elementsByTagName("office:automatic-styles");
	if (list.length() > 0) {
		QDomNode process = list.item(0);
		QDomNodeList items = process.childNodes();
		for (uint i=0; i<items.length(); ++i) {
			QDomNode node = items.item(i);
			processStyleNode(node);
		}
	}
}

void OdtHtml::processStyleNode(const QDomNode& node) {
	if (node.isElement() && (node.nodeName() == "style:style")) {
		QDomElement element = node.toElement();
		QString family = element.attribute("style:family", QString::null);
		QString name = element.attribute("style:name", QString::null);
		if (family == "text") {
			QDomNode child = element.firstChild();
			if ((!child.isNull()) && child.isElement() && 
				(child.nodeName() == "style:text-properties")) {
				QDomElement textPropertiesNode = child.toElement();
				int style;

				// Bold

				QString fontWeight = textPropertiesNode.attribute("fo:font-weight",
					QString::null);
				if (fontWeight == "bold") {
					style = style | TextStyle::Bold;
				}

				// Estilo

				QString fontStyle = textPropertiesNode.attribute("fo:font-style",
					QString::null);
				if (fontStyle == "italic") {
					style = style | TextStyle::Italic;
				}

				// Underline

				QString textUnderline = textPropertiesNode.attribute(
					"style:text-underline-style", QString::null);
				if (textUnderline == "solid") {
					style = style | TextStyle::Underline;
				}

				TextStyle* textStyle = new TextStyle();
				textStyle->style = style;
				mapStyles.insert(name, textStyle);
			}
		}		
	}
}

void OdtHtml::processOfficeText(const QDomElement& rootElement) {
	QDomNodeList list = rootElement.elementsByTagName("office:text");
	if (list.length() > 0) {
		QDomNode process = list.item(0);
		QDomNodeList items = process.childNodes();
		for (uint i=0; i<items.length(); ++i) {
			QDomNode node = items.item(i);
			processOfficeTextChild(node);
		}
	}
}

void OdtHtml::processOfficeTextChild(const QDomNode& node) { 
	if (node.isText()) {
		sw->writeCharacters(node.toText().data());
	} else if (node.isElement()) {
		QDomElement element = node.toElement();
		QString tagName;
		QXmlStreamAttributes attrs;
		QString tagNameOdt = element.tagName();
		QStack<XmlTag*> stack;
		if (tagNameOdt == "text:h") {
			QDomAttr attr = element.attributeNode("text:outline-level");
			if (!attr.isNull()) {
				XmlTag* xmltag = new XmlTag();
				xmltag->name = "h" + attr.value();
				stack.push(xmltag);
			}
		} else if (tagNameOdt == "text:p") {
			XmlTag* xmltag = new XmlTag();
			xmltag->name = "p";
			stack.push(xmltag);	
		} else if (tagNameOdt == "text:span") {
			QString styleName = element.attribute("text:style-name", QString::null);
			
			if (mapStyles.contains(styleName)) {
				TextStyle* textStyle = mapStyles.take(styleName);
				if (textStyle->style & TextStyle::Bold) {
					XmlTag* xmltag = new XmlTag();
					xmltag->name = "b";
					stack.push(xmltag);
				}

				if (textStyle->style & TextStyle::Italic) {
					XmlTag* xmltag = new XmlTag();
					xmltag->name = "i";
					stack.push(xmltag);
				}	

				if (textStyle->style & TextStyle::Underline) {
					XmlTag* xmltag = new XmlTag();
					xmltag->name = "u";
					stack.push(xmltag);
				}
			}			
		} else if (tagNameOdt == "text:a") {
			XmlTag* xmltag = new XmlTag();
			xmltag->name = "a";
			QDomAttr href = element.attributeNode("xlink:type");
			if (!href.isNull()) {
				xmltag->attrs.append("href", href.value());
			}
			stack.push(xmltag);			
		}

		int tagsCount = stack.size();
	
		// Escribe la apertura de los tags

		while (!stack.isEmpty()) {
			XmlTag* xmltag = stack.pop();
			sw->writeStartElement(xmltag->name);
			if (!xmltag->attrs.isEmpty()) {
				sw->writeAttributes(xmltag->attrs);
			}
			delete xmltag;
		}

		// Procesa el contenido anidado

		QDomNodeList childs = node.childNodes();
		for (uint i=0; i<childs.length(); ++i) {
			QDomNode node = childs.item(i);
			processOfficeTextChild(node);
		}

		// Escribe el cierre de los tags abiertos

		for (int i=0; i<tagsCount; ++i) {
			sw->writeEndElement();
		}
	}
}

int main(int argc, char *argv[]) {

    QApplication app(argc, argv);

/*    QWidget window;
    QVBoxLayout* mainLayout = new QVBoxLayout(&window);
    QLabel* label1 = new QLabel("One");
    QLabel* label2 = new QLabel("Two");
    mainLayout->addWidget(label1);
    mainLayout->addWidget(label2);
    window.show();*/


	QString fileName = argv[1];
	OdtHtml* odtHtml = new OdtHtml();

	try {
		odtHtml->convert(fileName);
		QString html = odtHtml->getHtml();
		
		QClipboard* clipboard = QApplication::clipboard();
		// clipboard->setText(html);
		QMimeData* mime = new QMimeData();
		mime->setText(html);
		clipboard->setMimeData(mime, QClipboard::Clipboard);

		/* QMessageBox::information(NULL, "Hello", clipboard->text(), 
			QMessageBox::Ok);  */

		QMessageBox::information(NULL, "HTML generado", html, 
			QMessageBox::Ok); 

	} catch (Exception ex) {
		QMessageBox::critical(NULL, "ERROR", ex.getMessage(), 
			QMessageBox::Ok);
	}

	delete odtHtml;

	// return app.exec(); 

	return 0;
}
